/**********************************************************************
File Name:            RFID_READER.c
Dev:                  Daniele De Santis
Email:                desantis@areasx.com
Web:                  http://www.areasx.com

Versione:             0.0
Data:                 13/01/11
Note Hardware:        RCM 3700 su SX3000
Note Software:
**********************************************************************/

#define DINBUFSIZE    511
#define DOUTBUFSIZE   511


unsigned long hstod(char *hexStr);
cofunc int rfid_reader();
void BoardInit(void);


typedef struct {
  unsigned long DATETIME;
  int CRC;
  char VALUE[11];
  unsigned long NUMBER;
} TAG;

TAG Rfid;

unsigned long hstod(char *hexStr) {
  int n;
  unsigned long value;
  int strsize;
  int dgt;
  strsize = strlen(hexStr);
  if(strsize>10) return 0;
  value = 0;
  for(n=0;n<=strsize;n++) {
    if(hexStr[n]=='\0') break;
    else if(hexStr[n] > 0x29 && hexStr[n] < 0x40 )
      dgt =hexStr[n] & 0x0f;
    else if(hexStr[n] >='a' && hexStr[n] <= 'f')
      dgt = (hexStr[n] & 0x0f) + 9;
    else if(hexStr[n] >='A' && hexStr[n] <= 'F')
      dgt = (hexStr[n] & 0x0f) + 9;
    else if(hexStr[n] == ' ')
      dgt = 0;
    else break;
    value = value | (unsigned long)dgt << ((strsize-n-1) << 2);
  }
  return value;
}

cofunc int rfid_reader(){
  unsigned int rx_byte;
  char  buffer[15];
  int   checksum_tmp;
  int   j;
  char  tmp[3];
  rx_byte=serDread(buffer,14,100);
  if (rx_byte>13) {
    buffer[rx_byte]=0;
    if(buffer[0]==0x02 && buffer[13]==0x03){
      memcpy(tmp,buffer+11, 2);
      tmp[2]=0;
      Rfid.CRC=(int)hstod(tmp);
      memcpy(Rfid.VALUE,buffer+1, 10);
      Rfid.VALUE[10]=0;
      checksum_tmp=0;
      for (j=0; j<(strlen(Rfid.VALUE)/2); j++){
        memcpy(tmp,Rfid.VALUE + (j*2), 2);
        tmp[2]=0;
        checksum_tmp=checksum_tmp^(int)hstod(tmp);
      }
      if(checksum_tmp==Rfid.CRC){
        memcpy(buffer,Rfid.VALUE+2, 8);
        buffer[8]=0;
        Rfid.NUMBER=hstod(buffer);
        return 1;
      } else
        return -1;//check sum check ERROR!

    }

  }
  return 0;
}


void BoardInit(void) {
  // Inizializzazione port PB0 in ingresso (pulsante)
  BitWrPortI(PBDDR, &PBDDRShadow, 0,0);
  // Inizializzazione port PB2 (Led 1)
  BitWrPortI(PBDDR, &PBDDRShadow, 1,2);
  // Inizializzazione port PE5 e PE7 (Led 2 e 3)
  WrPortI(PEFR, &PEFRShadow,0 );
  BitWrPortI(PEDDR, &PEDDRShadow, 1,5);
  BitWrPortI(PEDDR, &PEDDRShadow, 1,7);
  // Inizializzazione port PB3 PB4 PB5 PB7 (Analogica)
  BitWrPortI(PBDDR, &PBDDRShadow, 1,3);
  BitWrPortI(PBDDR, &PBDDRShadow, 1,4);
  BitWrPortI(PBDDR, &PBDDRShadow, 1,5);
  BitWrPortI(PBDDR, &PBDDRShadow, 0,7);
  // Inizializzazione seriale su PC0-PC1 (Ser D)
  BitWrPortI(PCFR, &PCFRShadow, 1,0);
  // Inizializzazione seriale su PC2-PC3 (Ser C)
  BitWrPortI(PCFR, &PCFRShadow, 1,2);
}

main() {

  unsigned int res;
  printf("*************************\nTest RFID reader \nSu SX3000 e RCM3700\n*************************\n\n");

  BoardInit();
  serDopen(9600);

  while (1) {
    costate {
      waitfor (DelaySec(1));
      wfd res=rfid_reader();
      if(res==1){
        printf("Passaggio RFID Tag\n");
        printf("Tag numero: %010lu\n", Rfid.NUMBER);
        printf("Tag valore: %s\n", Rfid.VALUE);
        printf("CRC card %d\n", Rfid.CRC);
        printf("------------------------------------\n");
      } else if(res==1)
        printf("Verifica CRC del Tag RFID fallita\n");
    }
  }
}